
#ifndef CLAMP_DSP_PLUGIN_H
#define CLAMP_DSP_PLUGIN_H

#include <AppKit.h>
#include <StorageKit.h>
#include <interface/View.h>

#ifndef CLAMP_PLUGIN_MSGS_H
 #include "Plugin_Messages.h"
#endif


// The following is for the GetFlags() return value
//  NO FLAGS FOR THE MOMENT!

// The class description of the object that will be returned when CL-Amp ask the AddOn (Plugin) for its object...
class DSPPlugin {
public:
// Used by CL-Amp. Don't bother about these functions!
	DSPPlugin(char *label, char *desc) { CLAmpHandler=NULL; Label= label; Description= desc; }
	virtual ~DSPPlugin() { }
	void SetHandler(BView *handler) { CLAmpHandler= handler; }

// =================================================================
// Here comes the functions you have to provide in your plugin code!

// No thread considerations with this function!
	virtual bool Init(long Frequency, short Channels, bool SampleIsOnly8Bits)=0;
	virtual void Cleanup()=0;

// These functions will run in thread A, dont let functions from A use data from B or vice verse...
	virtual bool About(bool Question)=0;
	virtual bool Prefs(bool Question)=0;
	virtual unsigned long GetFlags()=0;

// These functions will run in thread B, dont let functions from B use data from A or vice verse...
	// Process shall return the new size of Buff. The new size MUST be smaller than Size*2 and it
	// MUST be greater than Buff/2 !! The function can NOT fail! If something is wrong in your plugin,
	// just return Size and don't touch the buffer!
	virtual long Process(char *Buff, long Size)=0;
	virtual void Flush()=0;

// End of the functions you have to provide in your plugin code!
// =================================================================

// Help functions you can call to get data from the InputPlugin class itself
	const char *GetLabel() { return Label; }
	const char *GetDescription() { return Description; }
	BHandler *GetCLAmpHandler() { return CLAmpHandler; }
	void SendToCLAmp_ChangedFile(const char *FileName) { BMessage Msg(CLAMP_MSG_CHANGED); BLooper *l; if (CLAmpHandler && (l=CLAmpHandler->Looper())) { Msg.AddString(CLAMP_CHANGED_LABEL, FileName); l->PostMessage (&Msg, CLAmpHandler); } }
	void SendToCLAmp_AddFile(const char *FileName) { BMessage Msg(CLAMP_MSG_ADD); BLooper *l; if (CLAmpHandler && (l=CLAmpHandler->Looper())) { Msg.AddString(CLAMP_ADD_LABEL, FileName); l->PostMessage (&Msg, CLAmpHandler); } }
	void SendToCLAmp_DelFile(const char *FileName) { BMessage Msg(CLAMP_MSG_DEL); BLooper *l; if (CLAmpHandler && (l=CLAmpHandler->Looper())) { Msg.AddString(CLAMP_DEL_LABEL, FileName); l->PostMessage (&Msg, CLAmpHandler); } }
private:
	char *Label, *Description;
	BView *CLAmpHandler;
};

#define CURRENT_DSP_PLUGIN_VERSION  1
#define DSP_PLUGIN_VERSION_MASK	0x0fff

#endif
