#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <Bitmap.h>
#include <StopWatch.h>
#include <Autolock.h>

#include "MyScreenSaver.h"
#include "ConfigView.h"

extern "C" _EXPORT BScreenSaver *instantiate_screen_saver(BMessage *message, image_id image)
{
	return new MyScreenSaver(message, image);
}

MyScreenSaver::MyScreenSaver(BMessage *state, image_id image) :
	BScreenSaver(state, image),
	mImageId(image)
{
	y= x= 0;
	x_diff= y_diff= 1;
	remote= NULL;
	DecodeState(state);
	DispView= NULL;
	DispBmp= NULL;
}

MyScreenSaver::~MyScreenSaver()
{
}

void MyScreenSaver::DecodeState(BMessage *state)
{
	const char *str;
	if (state->FindString("NoRunText", &str) != B_OK)
		str = "CL-Amp is not running";
	strncpy (Config.NotRunningText, str, MAXLEN);
	if (state->FindString("NoPlayText", &str) != B_OK)
		str = "CL-Amp is not playing";
	strncpy (Config.NotPlayingText, str, MAXLEN);
}

char *MyScreenSaver::GetNotRunningText()
{
	BAutolock lock(ConfigLock);

	return strdup(Config.NotRunningText);
}

void MyScreenSaver::SetNotRunningText(const char *text)
{
	BAutolock lock(ConfigLock);

	strncpy (Config.NotRunningText, text, MAXLEN);
}

char *MyScreenSaver::GetNotPlayingText()
{
	BAutolock lock(ConfigLock);

	return strdup(Config.NotPlayingText);
}

void MyScreenSaver::SetNotPlayingText(const char *text)
{
	BAutolock lock(ConfigLock);

	strncpy (Config.NotPlayingText, text, MAXLEN);
}

image_id MyScreenSaver::GetImageId()
{
	return mImageId;
}

void MyScreenSaver::StartConfig(BView *view)
{
	BAutolock lock(ConfigLock);

	new ConfigView(view, this);
}

void MyScreenSaver::StopConfig()
{
}

status_t MyScreenSaver::SaveState(BMessage *state) const
{
	state->MakeEmpty();
	state->AddString("NoRunText", Config.NotRunningText);
	state->AddString("NoPlayText", Config.NotPlayingText);
	return B_OK;
}

status_t MyScreenSaver::StartSaver(BView *view, bool /*preview*/)
{
	BAutolock lock(ConfigLock);

	if (remote) delete remote;

	// Allocating a new Remote Control for CL-Amp
	remote= new CLAmp_FrontEnd;
	if (remote) {
		if (remote->InitCheck()) {
			int L=0;
			remote->StartRunning( false ); // Start running the remote control threads, but don't start CL-Amp if it's not already running
			while (++L<20 && !remote->Refresh()) snooze(20*1000); // Wait for a first info block from CL-Amp in a short time (otherwise we just continue)
		}
		else {
			//printf( "Error initializing CL-Amp frontend\n");
			delete remote, remote= NULL;
		}
	}

	srand(system_time());
	
	BScreen screen;
	ScreenHeight= int(screen.Frame().Height());
	ScreenWidth=  int(screen.Frame().Width());
	
	// To prevent the info text from always starting at the same place (= bad screen saver)
	// a random start position and moving direction is calculated here...
	x= (long)((float)ScreenWidth*(float)rand()/(float)RAND_MAX);
	y= (long)((float)ScreenHeight*(float)rand()/(float)RAND_MAX);
	if (rand()&1) x_diff= -x_diff;
	if (rand()&1) y_diff= -y_diff;

	// How often BeOS should ask us to redraw the screensaver view (call the Draw(...) function)
	SetTickSize(500*1000);
	
	// Allocating a bitmap and giving it an own view so we can draw more easily in it.
	// The bitmap will be used to draw in the actual view in order to prevent the screen from flickering while we draw...
	if (DispBmp)  delete DispBmp, DispBmp= NULL;
	DispView= NULL;  // The bitmap should delete it's children
	DispView= new BView(screen.Frame(), "DispView", B_FOLLOW_ALL_SIDES, 0);
	DispBmp= new BBitmap(screen.Frame(), B_RGB32, TRUE);
	if (DispBmp && DispView)
		DispBmp->AddChild(DispView);
	else {
		if (DispBmp)  delete DispBmp, DispBmp= NULL;
		if (DispView) delete DispView, DispView= NULL;
	}

	return B_OK;
}

void MyScreenSaver::StopSaver()
{
	BAutolock lock(ConfigLock);

	if (remote) delete remote;
	remote= NULL;
	if (DispBmp)  delete DispBmp, DispBmp= NULL;
	DispView= NULL;  // The bitmap should delete it's children
}

void MyScreenSaver::Draw(BView *view, int32 frame)
{
	#define MAXLEN 256
	static char TheText[MAXLEN];
	bool Playing= false, Fading= false, Paused= false;
	long PromilleReady= 0;
	long CurrTime=0, TotTime=0;
	
	// Lock the configuration data and if it's not possible simply return!
	BAutolock lock(ConfigLock);
	if (!lock.IsLocked())
		return;

	// If we have a double buffer to draw in, use it, otherwise use the actual output view
	BView *dispView= DispView? DispView : view;

	if (dispView->Window()->Lock()) {
		// A text to be displayed if nothing seems to work... :-)
		strncpy (TheText, "Memory problems??", MAXLEN);
		
		// Reading the information from CL-Amp
		if (remote) {
			remote->Refresh();
			if (remote->CLAmpIsRunning()){ 
				if ((Playing=remote->Playing()))
					strncpy (TheText, remote->Title(), MAXLEN);
				else
					strncpy (TheText, Config.NotPlayingText, MAXLEN);
			}
			else
				strncpy (TheText, Config.NotRunningText, MAXLEN);
			if (Playing) {
				PromilleReady= remote->Position();
			}
			CurrTime= remote->CurrTime()/1000;
			TotTime= remote->TotTime()/1000;
			Paused= remote->Paused();
			Fading= remote->Fading();
		}

		// If this is the first drawing we could set some default values...
		if (frame == 0) {
			dispView->SetLowColor(0,0,0);
			dispView->SetViewColor(B_TRANSPARENT_32_BIT);
			dispView->SetBlendingMode(B_CONSTANT_ALPHA, B_ALPHA_OVERLAY);
		}
		
		// The rest of this code is just to draw the the texts and info on to the screen.
		// I have not optimized or cleaned up this so don't take it too serious, please...
// ---  RED ALERT: MESSY CODE IS FOLLOWING!! :-)   ---
		static const int DefFontSize= 20;
		long MaxWidth= ScreenWidth/2;
		long MinWidth= ScreenWidth/4;
		long MinSize= 4;
		long MaxSize= 80;
		long Fork= (MaxSize-MinSize)/2;
		long CurrSize= MinSize+Fork;

		BFont font;
		font_height FH;
		dispView->GetFont(&font);
		do {
			font.SetSize(CurrSize);
			if (font.StringWidth(TheText)<MaxWidth)
				CurrSize+= Fork;
			Fork/= 2;
			if (Fork<1) Fork= 1;
			while (CurrSize>MinSize) {
				font.SetSize(CurrSize);
				if (font.StringWidth(TheText)<MaxWidth)
					break;
				else
					CurrSize-= Fork;
			}
		} while (Fork>1);
		if (CurrSize<MinSize) CurrSize= MinSize;
		if (CurrSize>MaxSize) CurrSize= MaxSize;
		font.SetSize(CurrSize);
		long TitleWidth= (long)font.StringWidth(TheText);
		font.SetFlags(font.Flags() & ~B_DISABLE_ANTIALIASING);
		dispView->SetFont(&font, B_FONT_SIZE);
		font.GetHeight(&FH);
		long YOffset= (long)FH.ascent+(long)FH.leading;
		long RectWidth= MAX(MinWidth, TitleWidth+4+(long)FH.leading*2);
		long RectHeight= (long)FH.ascent+(long)FH.descent+(long)FH.leading*2;
	
		font.SetSize(DefFontSize);
		font.GetHeight(&FH);
	
		long TotWidth= RectWidth;
		long TotHeight= RectHeight+(long)FH.ascent+(long)FH.descent+(long)FH.leading*2;
		long MaxX= ScreenWidth-TotWidth-1;
		long MaxY= ScreenHeight-TotHeight-1;

		x+= x_diff;
		if (x&1) y+= y_diff;
		if (x<0) x=0, x_diff= -x_diff;
		if (y<0) y=0, y_diff= -y_diff;
		if (x>MaxX) x=MaxX, x_diff= -x_diff;
		if (y>MaxY) y=MaxY, y_diff= -y_diff;

		dispView->SetHighColor(0,0,0);
		dispView->FillRect(dispView->Frame());
		dispView->SetHighColor(255,255,255);
		dispView->FillRect(BRect(x, y, x+TotWidth, y+TotHeight));
		dispView->SetHighColor(128,196,255);
		dispView->SetLowColor(255,255,255);
		dispView->StrokeRect(BRect(x, y, x+TotWidth, y+TotHeight));
		if (Playing) dispView->FillRect(BRect(x+3, y+2+RectHeight, x+2+(((RectWidth-2)*PromilleReady)/1000)-4, y+RectHeight+2+20-2));
		dispView->SetHighColor(255,0,0);
	
		dispView->DrawString(TheText, BPoint(x+2, y+YOffset));
		font.SetFlags(font.Flags() | B_DISABLE_ANTIALIASING);
		dispView->SetFont(&font, B_FONT_SIZE);
		YOffset= (long)FH.ascent+(long)FH.leading;

		// If CL-Amp is playing we draw some more info... (song length, remaining time and more...)
		if (Playing) {
			char Line[100];
			long low_y= y+YOffset+RectHeight;
			sprintf (Line, "%02ld:%02ld", CurrTime/60, CurrTime%60);
			dispView->DrawString(Line, BPoint(x+4, low_y));
			sprintf (Line, "-%02ld:%02ld", (TotTime-CurrTime)/60, (TotTime-CurrTime)%60);
			long TextWidth= (long)font.StringWidth(Line);
			dispView->DrawString(Line, BPoint(x+TotWidth-TextWidth-4, low_y));		
			if (Paused)
				sprintf (Line, ">Paused<");
			else if (Fading)
				sprintf (Line, ">Fading<");
			else
				sprintf (Line, ">%02ld:%02ld<", TotTime/60, TotTime%60);
			TextWidth= (long)font.StringWidth(Line);
			dispView->DrawString(Line, BPoint(x+TotWidth-TotWidth/2-TextWidth/2, low_y));
		}

		dispView->Sync();
		dispView->Window()->Unlock();
	
		// If we have the double buffer (the bitmap), draw it into the actual view
		if (DispBmp && view->Window()->Lock()) {
			view->DrawBitmap (DispBmp, view->Frame());
			view->Window()->Unlock();
		}
	}
}
